#include <sys.h>
#include <file.h>
#include <p270256b.h>
#include <graphics.h>

// ====================
// =====audio task=====
// ====================

// ===DMA setting===
// *addr ==> OC4RS
// trig TMR4 irq
// 1byte per event
// destination size is 1byte.
// src size is `size`

// DMA channel 1 and 2 for SD, channel 3 for keyboard, so we use channel 0 to audio.(No more DMA channel;)

void initdma(char *addr,unsigned size){
  DMACON[SET]    = 0x8000;
  DCH0CON[ORIG]  = 0x12;
  DCH0ECON[ORIG] = (19<<8)|(1<<4);
  DCH0SSA[ORIG]  = ((unsigned)addr) & 0x1FFFFFFF;
  DCH0DSA[ORIG]  = ((unsigned)OC4RS) & 0x1FFFFFFF;
  DCH0SSIZ[ORIG] = size;
  DCH0DSIZ[ORIG] = 1;
  DCH0CSIZ[ORIG] = 1;
  DCH0INT[CLR]   = 0x00FF00FF;
  DCH0CON[SET]   = 0x80;
}

#define CLRFLG DCH0INT[CLR] = 0x00FF00FF
#define ISHALFDONE DCH0INT[ORIG] & 0x40
#define ISALLDONE  DCH0INT[ORIG] & 0x80


// ===base timer===
// use timer4
// interval 32kHz

void setupbasetimer(void){
  T4CON[ORIG] = 0;
  TMR4[ORIG]  = 0;
// in text mode, (3.5795M*16)/32k=1790
// graphic mode, (3.5795M*15)/32k=1678
  PR4[ORIG]   = 1678;
  T4CON[SET]  = 0x8000;
}

// ===PWM settings===
// use timer3
// use OC4
// pwm max duty = 0xFF (8bit)
// output PB13

void setuppwm(void){
  RPB13R[ORIG]= 5;
  PR3[ORIG]   = 0x0200;
  TMR3[ORIG]  = 0;
  T3CON[SET]  = 0x8000;
  OC4CON[ORIG]= 0x000e;
  OC4CON[SET] = 0x8000;
}

char buff[512*8];
void initmusic(void){
  setuppwm();
  setupbasetimer();
  initdma(buff,sizeof(buff));
}

void stopmusic(void){
  T3CON[CLR] = 0x8000;
  T4CON[CLR] = 0x8000;
  DCH0CON[CLR] = 0xFF;
}

// Graphic buffer
// 1 screen and 1 frame size.
short gbuff[256*224/4+256*144/4];

int main(int argc,char **argv){  
  FILE *fp,*video;
  unsigned char palette[16*3];
  unsigned short *vramp = gbuff;

  fp = fopen("bgm.raw","r");
  video = fopen("video","r");
  if(fp==0||video==0){
    printf("file cannot open");
    return -1;
  }
  gVRAM = gbuff;
  set_graphmode(1);

  g_set_palette(0,128,128,128);

  initmusic();
  int i;
  int *bp = 0;
  int isread = 1;

// Return to quit.
  while(ps2keystatus[0xD]) asm("wait");
  while(isread){
    if(ISHALFDONE){
      bp = buff;
      CLRFLG;
    }else if(ISALLDONE){
      bp = buff + sizeof(buff)/2;
      CLRFLG;
    }
    if(bp){
      isread = fread(bp,1,sizeof(buff)/2,fp);bp = 0;
    }
    fread(palette,16*3,1,video);
    fread(gbuff+256*224/4,256*144/2,1,video);
    while(drawcount%2==0)asm("wait");
    while(drawcount%2!=0)asm("wait");
    for(i=0;i<16;i++){
      g_set_palette(i,palette[i*3+2],palette[i*3+0],palette[i*3+1]);
    }
    mem16copy(gbuff,gbuff+256*(224)/4,256*(144)/32);
    
    if(ps2keystatus[0xD])break;
  }
  wait60thsec(1);
  stopmusic();
  set_graphmode(0);

  return 0;
}